
"""import problems? -> make sure site.py is in sys.path, or set PYTHONHOME....
"""


import types, threading, javalib
from javalib import JobjectType

cacheJclass = {}
cacheJclassLock = threading.Lock()

def getJclass( name):
    try:
        return cacheJclass[ name]
    except KeyError:
        cacheJclassLock.acquire()
        try:
            if not cacheJclass.has_key( name):
                cacheJclass[ name] = Jclass( name)
        finally:
            cacheJclassLock.release()
        return cacheJclass[ name]

class Jclass:
    def __init__( self, name):
        sdict = self.__dict__
        sdict[ '_name_'] = name
        from throwable import JavaClassNotFoundException
        try:
            oclass = javalib.jclassForName( name)
        except JavaClassNotFoundException:
            # Class.forName() seems to fails when loading local classes
            # -> use JNI's findClass() if the SDK's Class.forName() fails
            import re
            oclass = javalib.jclass( re.sub( r'\.', '/', name))
        sdict[ '_jclass_'] = sdict[ '_jobject_'] = oclass
        sdict[ '_fieldDic_'] = {}
        sdict[ '_staticFieldDic_'] = {}
    def __nonzero__( self):
        return 1 # so that python does not query for __nonzero__ and __len__
    def __cmp__( self, other):
        return not self.equals( other)
    def __str__( self):
        return str( self._jobject_)
    def __call__( self, *args):
        jtypes, jargs = apply( jargconvert, args)
        for constr in self._constructors_[ len( args)]:
            if not jargCheckNot( jtypes, constr.args):
                break
        else:
            from throwable import TypeOverloadingError
            raise TypeOverloadingError( "No match on arg types", jtypes)
        return MakeJobject( constr( self._jclass_, jargs))
    def __getitem__( self, attr):
        try:
            return self._fieldDic_[ attr]
        except KeyError:
            try:
                fld = Jfield( self, attr)
            except AttributeError:
                fld = self._methods_[ attr]
            self._fieldDic_[ attr] = fld
            return fld
    def __setattr__( self, attr, value):
        raise TypeError( self)
    def __getattr__( self, attr):
	if attr == '__coerce__':
	    raise AttributeError
        elif attr in ('_methods_', '_staticMeth_'):
            mths = {}
            statics = {}
            for meth in self._jclass_.getMethods():
                resmth = ResolvedMethod( meth)
                mname = resmth.name
                if resmth.isStatic:
                    try:
                        mset = statics[ resmth.name]
                    except KeyError:
                        statics[ mname] = mset = JstaticMethod( resmth.name)
                else:
                    try:
                        mset = mths[ resmth.name]
                    except KeyError:
                        mths[ resmth.name] = mset = Jmethod( resmth.name)
                try:
                    margc = mset[ len( resmth.args)]
                except KeyError:
                    mset[ len( resmth.args)] = margc = []
                margc.append( resmth)
##            for meth in mths.values():
##                meth.sort()
##            for meth in statics.values():
##                meth.sort()
            self.__dict__[ '_methods_'] = mths
            self.__dict__[ '_staticMeth_'] = statics
            return getattr( self, attr)
        elif '_constructors_' == attr:
            mths = Jmethod( "<init>")
            for meth in self._jclass_.getConstructors():
                resconstr = ResolvedConstructor( meth)
                try:
                    margc = mths[ len( resconstr.args)]
                except KeyError:
                    mths[ len( resconstr.args)] = margc = []
                margc.append( resconstr)
            self.__dict__[ '_constructors_'] = mths
            return mths
        else:
            try:
                fld = self._staticFieldDic_[ attr]
            except KeyError:
                from throwable import JavaNoSuchFieldException
                try:
                    fld = JstaticField( self, attr)
                except JavaNoSuchFieldException:
                    try:
                        fld = self._staticMeth_[ attr]
                    except KeyError, err:
                        #if __debug__:
                            #print 'aaa', self._staticMeth_.keys()
                        raise AttributeError( err)
                self._staticFieldDic_[ attr] = fld
            return fld( self)

class Jfield:
    def __init__( self, cls, name):
        jfield = cls._jclass_.class_getField( name)
        if jfield.field_isStatic():
            raise ValueError( name, "is a static field")
        self.jfield = jfield
        self.fieldID = jfield.field_fromReflected()
    def __call__( self, jobject):
        return MakeJobject( self.fieldID.get( jobject._jobject_))
    def set( self, jobject, value):
        self.fieldID.set( jobject._jobject_,
                          isinstance( value, JobjectType)\
                            and res._jobject_ or res)

class JstaticField( Jfield):
    def __init__( self, cls, name):
        jfield = cls._jclass_.class_getField( name)
        if not jfield.field_isStatic():
            raise ValueError( name, "is not a static field")
        self.jfield = jfield
        self.fieldID = jfield.field_fromReflected()
    def __call__( self, jclass):
        return MakeJobject( self.fieldID.get( jclass._jclass_))
    def set( self, jclass, value):
        self.fieldID.set( jclass._jclass_,
                          isinstance( value, JobjectType)\
                            and res._jobject_ or res)

jTypeMap = None
jTypeMapLock = threading.RLock()

def jargconvert( *args):
    #if __debug__: print 'jargconvert', args
    global jTypeMap
    jargs = []
    jclasses = []
    for arg in args:
        if isinstance( arg, Jobject):
            jarg = arg._jobject_
            jargs.append( jarg)
            jclasses.append( jarg.getClass())
        else:
            if jTypeMap is None:
                jTypeMapLock.acquire()
                try:
                    if jTypeMap is None:
                        import java
                        jTypeMap = {
                            types.IntType: java.LongJType,
                            types.FloatType: java.DoubleJType,
                            types.NoneType: java.ObjectJType,
                            }
                finally:
                    jTypeMapLock.release()
            try:
                argType = jTypeMap[ type( arg)]
            except KeyError:
                jarg = javalib.jproxy( arg)
                jargs.append( jarg)
                jclasses.append( jarg.getClass())
            else:
                jargs.append( arg)
                jclasses.append( argType)
    #if __debug__: print 'return tuple( jclasses), tuple( jargs)',\
       #(tuple( jclasses), tuple( jargs))
    return tuple( jclasses), tuple( jargs)

class Jobject:
    def __init__( self, jobject):
        assert isinstance( jobject, JobjectType)
        self.__dict__[ '_jobject_'] = jobject
    def __nonzero__( self):
        return 1 # so that python does not query for __nonzero__ and __len__
    def __hash__( self):
        return self.hashCode()
    def __repr__( self):
        return repr( self._jobject_)
    def __str__( self):
        return str( self._jobject_)
    def __getattr__( self, attr):
	if attr == '__coerce__':
	    raise AttributeError
        elif attr == '_jclass_':
            result = self.__dict__[ attr]\
                     = getJclass( self._jobject_.className)
            return result
        elif attr == '_class_':
            result = self.__dict__[ attr] = self.getClass()
            return result
        else:
            try:
                attrobj = self._jclass_[ attr]
            except KeyError, err:
                raise AttributeError( err)
            return attrobj( self)
    def __setattr__( self, attr, val):
        self._jclass_[ attr].set( self, val)

IsStatic = 0x1
IsConstructor = 0x2

cmpjCache = {}
cmpjCacheLock = threading.RLock()
        
def jclassFullname( cls):
    result = ''
    while cls:
        result = str( cls) + result
        cls = cls.GetSuperClass()
    return result

def cmpjclass( cl1, cl2):
    try:
        n1 = cmpjCache[ str( cl1)]
    except KeyError:
        scl1 = str( cl1)
        cmpjCacheLock.acquire()
        try:
            if not cmpjCache.has_key( scl1):
                cmpjCache[ str( cl1)] = jclassFullname( cl1)
        finally:
            cmpjCacheLock.release()
        n1 = cmpjCache[ scl1]
    try:
        n2 = cmpjCache[ str( cl2)]
    except KeyError:
        scl2 = str( cl2)
        cmpjCacheLock.acquire()
        try:
            if not cmpjCache.has_key( scl2):
                cmpjCache[ str( cl2)] = jclassFullname( cl2)
        finally:
            cmpjCacheLock.release()
        n2 = cmpjCache[ scl2]
    return - cmp( n1, n2)

class ResolvedMethod:
    resMethAttr = {
        'args': lambda x: x.jmethod.method_getParameterTypes(),
        'className': lambda x: str( x.declaringClass),
        'declaringClass': lambda x: x.jmethod.method_getDeclaringClass(),
        'methodID': lambda x: x.declaringClass.getMethodID( x.name, x.sig,
                                                 x.isStatic and IsStatic or 0),
        'isStatic': lambda x: x.jmethod.method_isStatic(),
        'name': lambda x: str( x.jmethod),
        'returnType': lambda x: x.jmethod.method_getReturnType(),
        'sig': lambda x: '(%s)%s' % (reduce( lambda u, v: u + v.class_sig(),
                                             x.args, ''),
                                     x.returnType.class_sig()),
        }
    def __init__( self, jmethod):
        self.jmethod = jmethod
    def __call__( self, object, args):
        ##if __debug__: print 'gggggg', (self, self.methodID, object, args)
        if not isinstance( object, JobjectType):
            object = object._jobject_
        return MakeJobject( self.methodID( object, args))
    def __getattr__( self, attr):
        if '__coerce__' == attr:
            raise Attribute
        try:
            funattr = self.resMethAttr[ attr]
        except KeyError:
            raise AttributeError( attr)
        else:
            setattr( self, attr, funattr( self))
            return getattr( self, attr)
    def __cmp__( self, other):
        arg1 = self.args
        arg2 = other.args
        assert len( arg1) == len( arg2)
        for idx in range( len( arg1)):
            res = cmpjclass( arg1[ idx], arg2[ idx])
            if res:
                return res
        else:
            return 0
    def __repr__( self):
        return str( self)
    def __str__( self):
        return `(self.name, self.isStatic, self.sig, self.className)`

class ResolvedConstructor( ResolvedMethod):
    resMethAttr = {
        'args': lambda x: x.jmethod.constructor_getParameterTypes(),
        'className': lambda x: str( x.declaringClass),
        'declaringClass': lambda x: x.jmethod.constructor_getDeclaringClass(),
        'methodID': lambda x: x.declaringClass.getMethodID( x.name, x.sig,
                                                            IsConstructor),
        'name': lambda x: '<init>',
        'sig': lambda x: '(%s)V' % reduce( lambda u, v: u + v.class_sig(),
                                           x.args, ''),
        }
    def __str__( self):
        return `(self.name, IsConstructor, self.sig, self.className)`

simpleTypes = None

def jargCheckNot( jtypes, jdef):
    import java
    global simpleTypes
    if simpleTypes is None:
        simpleTypes = (java.BooleanJType, java.ByteJType, java.CharacterJType,
                       java.ShortJType, java.IntegerJType, java.LongJType)
    #if __debug__:
        #print 'jtypes = <%s>; jdef = <%s>' % (jtypes, jdef)
        #print 'LongJType=<%s> %s BooleanJType=<%s>'\
        #         % (LongJType, id( LongJType), BooleanJType)
    idx = 0
    for argTyp in jtypes:
        #if __debug__: print '\targTyp:', argTyp
        #if __debug__: print '\tjdef[idx]:', jdef[ idx]
        if argTyp is java.LongJType:
            #if __debug__:
                #print '--> jdef[ idx]', id( jdef[ idx])
                #print '--> intset', map( id, (BooleanJType, ByteJType,
                #        CharacterJType, ShortJType, IntegerJType, LongJType))
            if jdef[ idx] not in simpleTypes:
                break
        elif argTyp is java.DoubleJType:
            if jdef[ idx] not in (java.FloatJType, java.DoubleJType):
                break
        elif isinstance( argTyp, JobjectType):
            #if __debug__: print 'hhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhhh'
            if not argTyp.IsAssignable( jdef[ idx]):
                break
        else:
            break
        idx = idx + 1
    else:
        return None
    return (idx, jdef[ idx], argTyp)

from UserDict import UserDict
class Jmethod( UserDict):
    def __init__( self, name, args={}):
        self.name = name
        UserDict.__init__( self, args)
    def __call__( self, jobject):
        return JBoundMethod( self, jobject)
    def sort( self):
        for argmeth in self.values():
            argmeth.sort()

class JBoundMethod:
    def __init__( self, jmethod, jobject):
        self.jmethod = jmethod
        self.jobject = jobject
    def __repr__( self):
        return repr( (self.jobject.getClass(), self.jmethod))
    def __call__( self, *args):
        jtypes, jargs = apply( jargconvert, args)
        mths = self.jmethod[ len( args)]
        if not isinstance( mths, types.TupleType):
            mths.sort()
            self.jmethod[ len( args)] = mths = tuple( mths)
        #if __debug__:
            #print 'vvvvvvvvvvvvvvv:', mths
            #print 'wwwwwwwwwwwwwww:', jtypes
        for meth in mths:
            if not jargCheckNot( jtypes, meth.args):
                break
        else:
            from throwable import TypeOverloadingError
            raise TypeOverloadingError( "No match on arg types", jtypes, self)
##        if __debug__: print 'meth:', meth
        return meth( self.jobject, jargs)

class JstaticMethod( Jmethod):
    pass

class Jstring( Jobject):
    """java.lang.String
    """
    def __init__( self, arg):
        if not isinstance( arg, JobjectType):
            arg = javalib.jstring( arg)
        Jobject.__init__( self, arg)
    def __str__( self):
        return self._jobject_.string2UTF()

class Jarray( Jobject):
    """[]
    """
    def __init__( self, arg):
        assert( isinstance( arg, JobjectType))
        Jobject.__init__( self, arg)
    def __len__( self):
        return len( self._jobject_)
    def __getitem__( self, index):
        return MakeJobject( self._jobject_[ index])
    def __setitem__( self, index, value):
        if isinstance( value, JobjectType):
            value = value._jobject_
        self._jobject_[ index] = value

class Jlist( Jobject):
    """java.util.List
    """
    def __init__( self, arg):
        assert( isinstance( arg, JobjectType))
        Jobject.__init__( self, arg)
    def __len__( self):
        return self.size()
    def __getitem__( self, index):
        if isinstance( index, Slice):
            assert index.step is None or index.step == 1,\
                                              'extended slices not implemented'
            return self.subList( index.start, index.stop)
        else:
            if index < 0:
                index = index + len( self)
            return self.get( index)
    def __setitem__( self, index, value):
        if index < 0:
            index = index + len( self)
        return self.set( index, value)
    def __delitem__( self, index):
        if index < 0:
            index = index + len( self)
        return self.remove( index)
    def __contains__( self, item):
        return self.contains( item)
    def append( self, item):
        return self.add( item)
    def extend( self, jlist):
        return self.addAll( jlist)
    def index( self, item):
        return self.indexOf( item)
    def insert( self, index, item):
        self.add( index, item)
    def pop( self):
        return self.remove( len( self) - 1)
                      
class Jmap( Jobject):
    """java.util.Map
    """
    def __init__( self, arg):
        assert( isinstance( arg, JobjectType))
        Jobject.__init__( self, arg)
    def __len__( self):
        return self.size()
    def __getitem__( self, key):
        return self.get( key)
    def __setitem__( self, key, item):
        return self.put( key, item)
    def __delitem__( self, key):
        return self.remove( key)
    def keys( self):
        return self.keySet()
    def has_key( self, key):
        return self.containsKey( key)
    def get( self, key, default=None):
        res = Jobject.get( self, key)
        if res is None and not self.has_key( key):
            return default
        else:
            return res
    def update( self, map):
        self.putAll( map)

class Jstream( Jobject):
    """java.io.InputStream || java.io.OutputStream
    """
    def __init__( self, arg):
        assert( isinstance( arg, JobjectType))
        Jobject.__init__( self, arg)
    def read( self):
        raise NotImplementedError
    def write( self, data):
        raise NotImplementedError( 'transfer to byte[] and do write()')
    def flush( self):
        raise NotImplementedError
    def close( self):
        raise NotImplementedError
       
def MakeJobject( jobject):
    if isinstance( jobject, JobjectType):
        return javalib.makeClass( jobject)( jobject)
    else:
        return jobject
