/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is "Java-Python Extension (JPE)".
 * 
 * The Initial Developer of the Original Code is Frederic Bruno Giacometti.
 * Portions created by Frederic Bruno Giacometti are
 * Copyright (C) 2000-2001 Frederic Bruno Giacometti. All Rights Reserved.
 * 
 * Contributor(s): frederic.giacometti@arakne.com
 *
 * Acknowledgments:
 * Particular gratitude is expressed to the following parties for their
 * contributing support to the development of JPE:
 *     - Arakne SARL (France) and its staff (http://www.arakne.com);
 *       and in particular to Jean Hemmi, Marie-Therese Plaze and Didier Rano
 *     - The European Union through its funding of the IST Alice project
 *     - Dean Vucinic from Vrije Universiteit Brussel (Belgium),
 *       coordinator of the IST Alice project
 *     - iPhrase Technologies Inc. (Massachusetts, USA)
 */

package python;

/** Root class of Python exceptions in Java. */

public class PyException extends RuntimeException
{
  protected long exctype, exc, traceback;

  private PyException() {;}
  protected PyException( long lexctype, long lexc, long ltraceback)
  {
    exctype = lexctype;
    exc = lexc;
    traceback = ltraceback;
  }
  protected void finalize()
  {
    if (PyThreadState.lock_use)
      {
	PyThreadState pythread = PyThreadState.pyThread();
	pythread.lock();
	try
	  {
	    Javapy.Py_DECREF( 0, exctype);
	    Javapy.Py_DECREF( 0, exc);
	    Javapy.Py_DECREF( 0, traceback);
	  }
	finally { pythread.unlock(); }
      }
    else
      {
	long lpythread = PyThreadState.get();
	Javapy.Py_DECREF( lpythread, exctype);
	Javapy.Py_DECREF( lpythread, exc);
	Javapy.Py_DECREF( lpythread, traceback);
      }
  }
  public String toString()
  {
      long threadState = PyThreadState.get();
      System.err.println( Javapy.format_exception( threadState,
						   exctype, exc,
						   traceback));
      return "from Python\n" + Javapy.format_exception( threadState,
							exctype, exc,
							traceback);
  }
}
