/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is "Java-Python Extension (JPE)".
 * 
 * The Initial Developer of the Original Code is Frederic Bruno Giacometti.
 * Portions created by Frederic Bruno Giacometti are
 * Copyright (C) 2000-2001 Frederic Bruno Giacometti. All Rights Reserved.
 * 
 * Contributor(s): frederic.giacometti@arakne.com
 *
 * Acknowledgments:
 * Particular gratitude is expressed to the following parties for their
 * contributing support to the development of JPE:
 *     - Arakne SARL (France) and its staff (http://www.arakne.com);
 *       and in particular to Jean Hemmi, Marie-Therese Plaze and Didier Rano
 *     - The European Union through its funding of the IST Alice project
 *     - Dean Vucinic from Vrije Universiteit Brussel (Belgium),
 *       coordinator of the IST Alice project
 *     - iPhrase Technologies Inc. (Massachusetts, USA)
 */

package python;

import java.util.Map;
import java.util.List;
import java.util.Set;
import java.util.Collection;

public class PyMapping extends PyObject implements Map
{
  static public boolean check( PyObject obj)
  {
    return Javapy.PyMapping_Check( PyThreadState.get(), obj.pself);
  }
  protected PyMapping( long ptr) { super( ptr); }
  public PyMapping( PyObject obj)
  {
    super( obj.pself);
    if (!check( this)) throw new ClassCastException( //should Py_DECREF ?
	                   "Mapping protocol not supported by Python object");
  }
  public boolean equals( Object obj)
  {
    throw new UnsupportedOperationException( "PyMapping::equals( Object)");
  }
  public int size()
  {
    long res;
    res = Javapy.PyMapping_Length( PyThreadState.get(), pself);
    return res < Integer.MAX_VALUE ? (int)res : Integer.MAX_VALUE;
  }
  public boolean isEmpty() { return 0 != size(); }
  //Map implementation
  public boolean containsKey( PyObject key)
  {
    return Javapy.PyMapping_HasKey( PyThreadState.get(), pself, key.pself);
  }
  public boolean containsKey( Object key)
  {
    return containsValue( asPython( key));
  }
  public PyObject keys() { return (PyObject)callMethod( "keys"); }

  public boolean containsValue( Object value)
  {
    throw
	new UnsupportedOperationException( "PyObject::containsValue( Object)");
  }

  public Object put( Object key, Object val)
  {
    PyObject pykey = asPython( key);
    Object result;
    
    result = containsKey( pykey) ? get( key) : null;
    setItem( pykey, asPython( val));
    return result;
  }

  public Object get( PyObject key)
  {
      try
	{
	  return Javapy.ptr2object(
				Javapy.PyMapping_GetItem( PyThreadState.get(),
							  pself,
							  key.pself));
	}
      catch (KeyError e) {return null;}
  }

  public Object get( Object key)
  {
    return get( asPython( key));
  }
  public Object remove( PyObject key)
  {
    Object result = containsKey( key) ? get( key) : null;
    Javapy.PyMapping_DelItem( PyThreadState.get(), pself, key.pself);
    return result;
  }
  public Object remove( Object key)
  {
    return remove( asPython( key));
  }
  public void putAll( Map t)
  {
    throw new UnsupportedOperationException( "PyMapping::putAll(Map)");
  }
  public Set keySet()
  {
    throw new UnsupportedOperationException( "PyMapping::keySet()");
  }
  public Set entrySet()
  {
    throw new UnsupportedOperationException( "PyMapping::entrySet()");
  }
  public Collection values()
  {
    throw new UnsupportedOperationException( "PyMapping::values()");
  }
}
