/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is "Java-Python Extension (JPE)".
 * 
 * The Initial Developer of the Original Code is Frederic Bruno Giacometti.
 * Portions created by Frederic Bruno Giacometti are
 * Copyright (C) 2000-2001 Frederic Bruno Giacometti. All Rights Reserved.
 * 
 * Contributor(s): frederic.giacometti@arakne.com
 *
 * Acknowledgments:
 * Particular gratitude is expressed to the following parties for their
 * contributing support to the development of JPE:
 *     - Arakne SARL (France) and its staff (http://www.arakne.com);
 *       and in particular to Jean Hemmi, Marie-Therese Plaze and Didier Rano
 *     - The European Union through its funding of the IST Alice project
 *     - Dean Vucinic from Vrije Universiteit Brussel (Belgium),
 *       coordinator of the IST Alice project
 *     - iPhrase Technologies Inc. (Massachusetts, USA)
 */

package python;

import java.lang.reflect.Array;

/** Utility for executing Python files and strings in the current Python
 * interpreter.
 * The same 'global' Python dictionary is reused across calls.
 */
public class PyRun
{
  private PyRun() {;}

  static private final PyModule runmodule = new PyModule( "java.run");

  /** evaluate a Python expression or bytecode
   * @param expression Python expresion or bytecode
   * @return result of the evaluation
   * @throws python.PyException
   */
  public static Object eval( String expression)
  {
    return runmodule.callMethod( "runeval", expression);
  }

  /** execute a Python expression or bytecode
   * @param expression Python expresion or bytecode
   * @throws python.PyException
   */
  public static void exec( String expression)
  {
    runmodule.callMethod( "runexec", expression);
  }

  /** execute the content of a Python file
   * @param filename local or absolute path of the Python file to execute
   * @throws python.PyException
   */
  public static Object execfile( String filename)
  {
    return runmodule.callMethod( "runexecfile", filename);
  }

  /** execute the content of a Python file
   * @param filename local or absolute path of the Python file to execute
   * @param args Python sys.argv values for the execution - to be implemented
   * @throws python.PyException
   */
  public static Object execfile( String filename, String args[])
  {
    if (1 < args.length)
      System.err.println( "Warning PyRun.execfile(): "
			  + "use of args not implemented - args ignored");
    return runmodule.callMethod( "runexecfile", filename); //, args);
  }

  /** command line utility for calling PyRun.execfile()
   * <br>will be augmented for handling all python options
   * @params args local or absolute path of the Python file to execute
   */
  public static void main( String args[])
  {
    execfile( args[ 0], args);
  }
}
